@@ -2,6 +2,8 @@ import numpy as np
 
 import pyzm.ml.face_train as train
 import face_recognition
+import dlib
+
 import sys
 import os
 import cv2
@@ -15,20 +17,41 @@ import datetime
 from pyzm.helpers.Base import Base
 # Class to handle face recognition
 import portalocker
+import re
+from pyzm.helpers.Media import MediaStream
+
 
 class Face(Base):
     def __init__(self, logger=None, options={},upsample_times=1, num_jitters=0, model='hog'):
         super().__init__(logger)
 
+        if dlib.DLIB_USE_CUDA and dlib.cuda.get_num_devices() >=1 :
+            self.processor = 'gpu'
+        else:
+            self.processor = 'cpu'
+
         self.logger.Debug(
             1,'Initializing face recognition with model:{} upsample:{}, jitters:{}'
             .format(model, upsample_times, num_jitters))
 
         self.upsample_times = upsample_times
         self.num_jitters = num_jitters
-        self.model = model
+        if options.get('face_model'):
+            self.face_model = options.get('face_model')
+        else:
+            self.face_model = model
+       
         self.knn = None
         self.options = options
+        self.is_locked = False
+
+        self.lock_maximum=options.get(self.processor+'_max_processes') or 1
+        self.lock_timeout = options.get(self.processor+'_max_lock_wait') or 120
+        
+        self.lock_name='pyzm_'+self.processor+'_lock'
+        self.logger.Debug (2,f'Semaphore: max:{self.lock_maximum}, name:{self.lock_name}, timeout:{self.lock_timeout}')
+        self.lock = portalocker.BoundedSemaphore(maximum=self.lock_maximum, name=self.lock_name,timeout=self.lock_timeout)
+        
 
         if self.options.get('face_detection_framework') != 'dlib' and self.options.get('face_recognition_framework') != 'dlib':
             raise ValueError ('Error: As of now,only dlib is supported for face detection and recognition. Unkown {}/{}'.format(self.options.get('face_detection_framework'),self.options.get('face_recognition_framework')))
@@ -68,6 +91,31 @@ class Face(Base):
         except Exception as e:
             self.logger.Error ('Error loading KNN model: {}'.format(e))
 
+
+    def acquire_lock(self):
+        if self.is_locked:
+            self.logger.Debug (1, '{} Lock already acquired'.format(self.lock_name))
+            return
+        try:
+            self.logger.Debug (1,f'Waiting for {self.processor} lock...')
+            self.lock.acquire()
+            self.logger.Debug (1,f'Got {self.processor} lock for initialization...')
+           
+            self.lock.release()
+            self.logger.Debug(1,'init lock released')
+        except portalocker.AlreadyLocked:
+            self.logger.Error ('Timeout waiting for {} lock for {} seconds'.format(self.processor, self.lock_timeout))
+            raise ValueError ('Timeout waiting for {} lock for {} seconds'.format(self.processor, self.lock_timeout))
+
+
+    def release_lock(self):
+        if not self.is_locked:
+            self.logger.Debug (1, '{} Lock already released'.format(self.lock_name))
+            return
+        self.lock.release()
+        self.is_locked = False
+        self.logger.Debug (1,'Released lock')
+
     def get_classes(self):
         if self.knn:
             return self.knn.classes_
@@ -84,6 +132,8 @@ class Face(Base):
             rects.append([left, top, right, bottom])
         return rects
 
+    
+
     def detect(self, image):
 
         Height, Width = image.shape[:2]
@@ -99,11 +149,14 @@ class Face(Base):
         #rgb_image = image
 
         # Find all the faces and face encodings in the target image
+        print (self.options)
+        if self.options.get('auto_lock',True):
+            self.acquire_lock()
 
         start = datetime.datetime.now()
         face_locations = face_recognition.face_locations(
             rgb_image,
-            model=self.model,
+            model=self.face_model,
             number_of_times_to_upsample=self.upsample_times)
 
         diff_time = (datetime.datetime.now() - start).microseconds / 1000
@@ -114,6 +167,10 @@ class Face(Base):
             rgb_image,
             known_face_locations=face_locations,
             num_jitters=self.num_jitters)
+        
+        if self.options.get('auto_lock',True):
+            self.release_lock()
+
         diff_time = (datetime.datetime.now() - start).microseconds / 1000
         self.logger.Debug(
             1,'Computing face recognition distances took {} milliseconds'.format(
@@ -152,7 +209,7 @@ class Face(Base):
 
         for pred, loc, rec in zip(prediction_labels,
                                   face_locations, are_matches):
-            label = pred if rec else self.options.get('unknown_face_name')
+            label = pred if rec else self.options.get('unknown_face_name', 'unknown')
             if not rec and self.options.get('save_unknown_faces') == 'yes':
                 h, w, c = image.shape
                 x1 = max(loc[3] - self.options.get('save_unknown_faces_leeway_pixels'),
@@ -176,7 +233,7 @@ class Face(Base):
                             self.options.get('save_unknown_faces_leeway_pixels'), unf))
                 cv2.imwrite(unf, crop_img)
 
-            matched_face_rects.append((loc[3], loc[0], loc[1], loc[2]))
+            matched_face_rects.append([loc[3], loc[0], loc[1], loc[2]])
             matched_face_names.append(label)
             conf.append(1)
 
